<?php
if (realpath(__FILE__) == realpath($_SERVER['SCRIPT_FILENAME'])) {
    exit(esc_html__('Please don\'t access this file directly.', 'GFCCGA'));
}

class GFCCGA_CreateCoupon_Cls {

    public function __construct($args = array()) {

        // set our default arguments, parse against the provided arguments, and store for use throughout the class
        $this->_args = wp_parse_args($args, array(
            'form_id'               => false,
            'source_field_id'       => false,
            'plugin'                => 'gf',
            'amount'                => 0,
            'type'                  => '',
            'meta'                  => array(
                'max_uses'          => '',
            )
        ));

        // make sure we're running the required minimum version of Gravity Forms
        if (!property_exists('GFCommon', 'version') || !version_compare(GFCommon::$version, '1.8', '>=')) {
            return;
        }

        add_action('gform_after_submission', array($this, 'create_coupon'), 10, 2);
    }

    ## create coupon
    public function create_coupon($entry, $form) {

        if (!$this->is_applicable_form($form)) {
            return;
        }

        $coupon_code = rgar($entry, $this->_args['source_field_id']);
        $coupon_name = $coupon_code;
        $amount      = $this->_args['amount'];
        $type        = $this->_args['type'];

        $plugin_func = array($this, sprintf('create_coupon_%s', $this->_args['plugin']));

        $coupon_data = array();
        $coupon_data['coupon_name'] = $coupon_name;
        $coupon_data['coupon_code'] = $coupon_code;
        $coupon_data['amount'] = $amount;
        $coupon_data['type'] = $type;
        $coupon_data['coupon_limit'] = $this->_args['meta']['max_uses'];

        if (is_callable($plugin_func)) {
            call_user_func($plugin_func, $entry, $form, $coupon_data);
        }
    }

    ## save the coupon code in the database
    public function create_coupon_gf($entry, $form, $coupon_data) {

        if (!class_exists('GFCoupons')) {
            return;
        }

        // load GF Coupons data.php file
        if (is_callable('gf_coupons')) {
            gf_coupons()->get_config(array('id' => 0), false);
        } else {
            /** @noinspection PhpDynamicAsStaticMethodCallInspection */
            GFCoupons::get_config(array('id' => 0), false);
        }

        $meta = wp_parse_args($this->_args['meta'], array(
            'form_id'           => false,
            'coupon_name'       => $coupon_data['coupon_name'],
            'coupon_code'       => strtoupper($coupon_data['coupon_code']),
            'coupon_type'       => $coupon_data['type'], // 'flat', 'percentage'
            'coupon_amount'     => $coupon_data['amount'],
            'coupon_start'      => '', // MM/DD/YYYY
            'coupon_expiration' => '', // MM/DD/YYYY
            'coupon_limit'      => false,
            'coupon_stackable'  => false,
            'max_uses'          => $coupon_data['coupon_limit'],
        ));

        $form_id = $meta['form_id'] ? $meta['form_id'] : 0;
        unset($meta['form_id']);

        foreach ($meta as $key => $value) {
            if ($value instanceof Closure && is_callable($value)) {
                $meta[$key] = call_user_func($value, $entry, $form, $this);
            }
        }

        if (is_callable('gf_coupons')) {
            $meta['gravityForm']      = $form_id ? $form_id : 0;
            $meta['couponName']       = $meta['coupon_name'];
            $meta['couponCode']       = $meta['coupon_code'];
            $meta['couponAmountType'] = $meta['coupon_type'];
            $meta['couponAmount']     = $meta['coupon_amount'];
            $meta['startDate']        = $meta['coupon_start'];
            $meta['endDate']          = $meta['coupon_expiration'];
            $meta['usageLimit']       = $meta['coupon_limit'];
            $meta['isStackable']      = $meta['coupon_stackable'];
            $meta['usageCount']       = 0;
            gf_coupons()->insert_feed($form_id, true, $meta);
        } else {
            /** @noinspection PhpUndefinedClassInspection */
            GFCouponsData::update_feed(0, $form_id, true, $meta);
        }
    }

    ## check if form is applicable
    function is_applicable_form($form) {
        $form_id = isset($form['id']) ? $form['id'] : $form;
        return $form_id == $this->_args['form_id'];
    }
}