<?php
if (realpath(__FILE__) == realpath($_SERVER['SCRIPT_FILENAME'])) {
    exit(esc_html__('Please don\'t access this file directly.', 'GFCCGA'));
}

class GFO_Coupons_Exclude_Shipping {

    protected static $is_script_output = false;
    public static $shipping_price = null;

    public function __construct() {

        ## do version check in the init to make sure if GF is going to be loaded, it is already loaded
        add_action( 'init', array( $this, 'init' ) );

    }

    function init() {

        $has_gravity_forms = property_exists( 'GFCommon', 'version' ) && version_compare( GFCommon::$version, '1.8', '>=' );
        $has_gf_coupons = class_exists( 'GFCoupons' );

        ## make sure we're running the required minimum version of Gravity Forms and GF Coupons
        if( ! $has_gravity_forms || ! $has_gf_coupons ) {
            return;
        }

        ## time for hooks
        add_filter( 'gform_pre_render', array( $this, 'load_form_script' ) );

        add_action( 'gform_product_info', array( $this, 'stash_shipping_price' ) );
        add_filter( 'gform_coupons_discount_amount', array( $this, 'modify_coupon_discount_amount' ), 10, 3 );

    }

    function load_form_script( $form ) {

        if( $this->is_applicable_form( $form ) && ! self::$is_script_output ) {
            $this->output_script();
        }

        return $form;
    }

    function output_script() {
        ?>

        <script type="text/javascript">

            ( function( $ ) {

                gform.addFilter( 'gform_coupons_discount_amount', function( discount, couponType, couponAmount, price, totalDiscount ) {

                    var formId = document.getElementsByName('gform_submit')[0].value,
	                    price  = price - gformGetShippingPrice( formId );

                    if( couponType == 'percentage' ) {
                        discount = price * Number( ( couponAmount / 100 ) );
                    } else if( couponType == 'flat' ) {
	                    discount = Number( couponAmount );
	                    if( discount > price ) {
		                    discount = price;
	                    }
                    }

                    return discount;
                } );

            } )( jQuery );

        </script>

        <?php

        self::$is_script_output = true;

    }

    function stash_shipping_price( $product_data ) {
        self::$shipping_price = $product_data['shipping']['price'];
        return $product_data;
    }

    function modify_coupon_discount_amount( $discount, $coupon, $price ) {

        if( ! self::$shipping_price ) {
            return $discount;
        }

	    $price    = $price - self::$shipping_price;
	    $currency = new RGCurrency( GFCommon::get_currency() );
	    $amount   = $currency->to_number( $coupon['amount'] );

        if( $coupon['type'] == 'percentage' ) {
            $discount = $price * ( $amount / 100 );
        } else if( $coupon['type'] == 'flat' ) {
	        $discount = $amount;
	        if( $discount > $price ) {
		        $discount = $price;
	        }
        }

        return $discount;
    }

    function is_applicable_form( $form ) {

        $coupon_fields = GFCommon::get_fields_by_type( $form, array( 'coupon' ) );

        return ! empty( $coupon_fields );
    }

}
