<?php
/*
Plugin Name: Auto Coupon Generate for Gravity Forms
Description: Allow users to create their own custom Coupon code for the Gravity forms
Author: krishnagajera
Version: 1.0.2
Text Domain: gf-auto-coupon-generate
*/

## define the plugin folder url
define('GFCCGA_PLUGIN_URL', plugin_dir_url(__FILE__));

## define the plugin folder dir
define('GFCCGA_PLUGIN_DIR', plugin_dir_path(__FILE__));

require_once(GFCCGA_PLUGIN_DIR . "config.php");
require_once(GFCCGA_PLUGIN_DIR . "functions.php");
require_once(GFCCGA_PLUGIN_DIR . "GFCCGA_CreateCoupon_Cls.php");

## Generates and returns the code
add_filter('gform_field_value_uuid', 'GFCCGA_generate_unique_code');
function GFCCGA_generate_unique_code()
{
    $length = GFCCGA_CC_CODE_LENGTH;
    $chars = GFCCGA_CC_CODE_CHARS;
    $prefix = GFCCGA_CC_CODE_PREFIX;

    $unique = '';
    $chars_length = strlen($chars) - 1;

    for ($i = 0; $i < $length; $i++) {
        $unique .= $chars[rand(0, $chars_length)];
    }

    do {
        $unique = $prefix . str_shuffle($unique);
    } while (!GFCCGA_check_unique_code($unique));

    return $unique;
}

## Checks to make sure the code generated is unique (not already in use)
function GFCCGA_check_unique_code($unique)
{
    global $wpdb;

    $table = $wpdb->prefix . 'rg_lead_detail';
    $form_id = GFCCGA_CC_FORM_ID; // update to the form ID your unique id field belongs to
    $field_id = GFCCGA_CC_SOURCE_FIELD_ID; // update to the field ID your unique id is being prepopulated in
    $result = $wpdb->get_var( $wpdb->prepare( "SELECT value FROM " . $table . " WHERE form_id = %d AND field_number = %s AND value = %s", $form_id, $field_id, $unique ) );

    if (empty($result)) {
        return true;
    } else return false;
}

new GFCCGA_CreateCoupon_Cls(
    array(
        'form_id'               => GFCCGA_CC_FORM_ID,
        'source_field_id'       => GFCCGA_CC_SOURCE_FIELD_ID,
        'amount'                => GFCCGA_CC_DISCOUNT_AMOUNT,
        'type'                  => GFCCGA_CC_DISCOUNT_TYPE,
        'meta'                  => array(
            'form_id'           => GFCCGA_CC_FORM_ID_REG,
            'excluded_products' => array(GFCCGA_CC_EXCLUDE_PRODUCTS),
            'expiration'        => GFCCGA_CC_EXP_DATE,
            'is_not_global'     => 'not_global',
            'is_single_use'     => GFCCGA_CC_SINGLE_USE,
            'max_uses'          => GFCCGA_CC_MAX_USES,
            'min_price'         => GFCCGA_CC_MIN_PRICE,
            'product_condition' => GFCCGA_CC_REQS_CONDITION,
            'product_reqs'      => array(GFCCGA_CC_PRODUCT_REQS),
            'start'             => GFCCGA_CC_START_DATE,
        )
    )
);

require_once(GFCCGA_PLUGIN_DIR . "exclude_shipping_Cls.php");
new GFO_Coupons_Exclude_Shipping();
